@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,12 +10,11 @@ from typing import List, Optional
 
 from typing_extensions import Final
 
-from .. import log
+from .. import command_arguments, log
 from ..analysis_directory import AnalysisDirectory, resolve_analysis_directory
 from ..configuration import Configuration
 from ..filesystem import readable_directory, writable_directory
 from .check import Check
-from .command import CommandArguments
 
 
 class Analyze(Check):
@@ -23,10 +22,10 @@ class Analyze(Check):
 
     def __init__(
         self,
-        command_arguments: CommandArguments,
+        command_arguments: command_arguments.CommandArguments,
         original_directory: str,
         *,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
         analysis: str,
         taint_models_path: List[str],
@@ -58,11 +57,11 @@ class Analyze(Check):
     def from_arguments(
         arguments: argparse.Namespace,
         original_directory: str,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
     ) -> "Analyze":
         return Analyze(
-            CommandArguments.from_arguments(arguments),
+            command_arguments.CommandArguments.from_arguments(arguments),
             original_directory,
             configuration=configuration,
             analysis_directory=analysis_directory,
@@ -115,15 +114,13 @@ class Analyze(Check):
 
     def generate_analysis_directory(self) -> AnalysisDirectory:
         return resolve_analysis_directory(
-            self._source_directories,
-            self._targets,
+            self._command_arguments.source_directories,
+            self._command_arguments.targets,
             self._configuration,
             self._original_directory,
-            self._project_root,
-            filter_directory=self._filter_directory,
-            use_buck_builder=self._use_buck_builder,
-            debug=self._debug,
-            buck_mode=self._buck_mode,
+            self._configuration.project_root,
+            filter_directory=self._command_arguments.filter_directory,
+            buck_mode=self._command_arguments.buck_mode,
             isolate=True,
         )
 
