@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,9 +8,10 @@ import logging
 from logging import Logger
 from typing import List, Optional
 
+from .. import command_arguments
 from ..analysis_directory import AnalysisDirectory, resolve_analysis_directory
 from ..configuration import Configuration
-from .command import CommandArguments, ExitCode, typeshed_search_path
+from .command import ExitCode, typeshed_search_path
 from .reporting import Reporting
 
 
@@ -22,10 +23,10 @@ class Check(Reporting):
 
     def __init__(
         self,
-        command_arguments: CommandArguments,
+        command_arguments: command_arguments.CommandArguments,
         original_directory: str,
         *,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
     ) -> None:
         super(Check, self).__init__(
@@ -36,11 +37,11 @@ class Check(Reporting):
     def from_arguments(
         arguments: argparse.Namespace,
         original_directory: str,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
     ) -> "Check":
         return Check(
-            CommandArguments.from_arguments(arguments),
+            command_arguments.CommandArguments.from_arguments(arguments),
             original_directory,
             configuration=configuration,
             analysis_directory=analysis_directory,
@@ -58,15 +59,13 @@ class Check(Reporting):
 
     def generate_analysis_directory(self) -> AnalysisDirectory:
         return resolve_analysis_directory(
-            self._source_directories,
-            self._targets,
+            self._command_arguments.source_directories,
+            self._command_arguments.targets,
             self._configuration,
             self._original_directory,
-            self._project_root,
-            filter_directory=self._filter_directory,
-            use_buck_builder=self._use_buck_builder,
-            debug=self._debug,
-            buck_mode=self._buck_mode,
+            self._configuration.project_root,
+            filter_directory=self._command_arguments.filter_directory,
+            buck_mode=self._command_arguments.buck_mode,
             isolate=True,
         )
 
@@ -74,10 +73,8 @@ class Check(Reporting):
         flags = super()._flags()
         filter_directories = self._get_directories_to_analyze()
         if len(filter_directories):
-            # pyre-fixme[6]: Expected `Iterable[Variable[_LT (bound to
-            #  _SupportsLessThan)]]` for 1st param but got `Set[str]`.
             flags.extend(["-filter-directories", ";".join(sorted(filter_directories))])
-        flags.extend(["-workers", str(self._number_of_workers)])
+        flags.extend(["-workers", str(self._configuration.number_of_workers)])
         search_path = self._configuration.search_path + typeshed_search_path(
             self._configuration.typeshed
         )
