@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -14,25 +14,16 @@ import signal
 import subprocess
 import threading
 from abc import ABC, abstractmethod
-from dataclasses import dataclass
 from pathlib import Path
 from typing import IO, Iterable, List, Optional
 
-from typing_extensions import Final
-
-from .. import json_rpc, log, recently_used_configurations, terminal
+from .. import command_arguments, json_rpc, log, recently_used_configurations, terminal
 from ..analysis_directory import AnalysisDirectory, resolve_analysis_directory
 from ..configuration import Configuration
 from ..exceptions import EnvironmentException
-from ..filesystem import readable_directory, remove_if_exists, translate_path
-from ..find_directories import (
-    LOCAL_CONFIGURATION_FILE,
-    find_local_root,
-    find_project_root,
-)
+from ..filesystem import readable_directory, remove_if_exists
 from ..log import StreamLogger
 from ..process import Process
-from ..resources import LOG_DIRECTORY, find_log_directory
 from ..socket_connection import SocketConnection, SocketException
 
 
@@ -57,6 +48,8 @@ class ExitCode(enum.IntEnum):
     FOUND_ERRORS = 1
     FAILURE = 2
     BUCK_ERROR = 3
+    SERVER_NOT_FOUND = 4
+    INCONSISTENT_SERVER = 5
     # If the process exited due to a signal, this will be the negative signal number.
     SIGSEGV = -signal.SIGSEGV
 
@@ -113,8 +106,6 @@ def typeshed_search_path(typeshed_root: str) -> List[str]:
             continue
 
         # Always prefer newer version over older version
-        # pyre-fixme[6]: Expected `Iterable[Variable[_LT (bound to
-        #  _SupportsLessThan)]]` for 1st param but got `List[str]`.
         version_names = sorted(os.listdir(typeshed_subdirectory), reverse=True)
         for version_name in version_names:
             # Anything under 2/ or 2.x is unusable for Pyre
@@ -145,161 +136,13 @@ def executable_file(file_path: str) -> str:
     return file_path
 
 
-@dataclass(frozen=True)
-class CommandArguments:
-    local_configuration: Optional[str]
-    version: bool
-    debug: bool
-    sequential: bool
-    strict: bool
-    additional_checks: List[str]
-    show_error_traces: bool
-    output: str
-    enable_profiling: bool
-    enable_memory_profiling: bool
-    noninteractive: bool
-    hide_parse_errors: bool
-    logging_sections: Optional[str]
-    log_identifier: str
-    logger: Optional[str]
-    formatter: List[str]
-    targets: List[str]
-    use_buck_builder: Optional[bool]
-    source_directories: List[str]
-    filter_directory: Optional[str]
-    buck_mode: Optional[str]
-    no_saved_state: bool
-    search_path: List[str]
-    binary: str
-    buck_builder_binary: Optional[str]
-    exclude: List[str]
-    typeshed: str
-    save_initial_state_to: Optional[str]
-    load_initial_state_from: Optional[str]
-    changed_files_path: Optional[str]
-    saved_state_project: Optional[str]
-    dot_pyre_directory: Optional[Path]
-    features: Optional[str]
-
-    @staticmethod
-    def from_arguments(arguments: argparse.Namespace) -> "CommandArguments":
-        return CommandArguments(
-            local_configuration=arguments.local_configuration,
-            version=arguments.version,
-            debug=arguments.debug,
-            sequential=arguments.sequential,
-            strict=arguments.strict,
-            additional_checks=arguments.additional_check,
-            show_error_traces=arguments.show_error_traces,
-            output=arguments.output,
-            enable_profiling=arguments.enable_profiling,
-            enable_memory_profiling=arguments.enable_memory_profiling,
-            noninteractive=arguments.noninteractive,
-            hide_parse_errors=arguments.hide_parse_errors,
-            logging_sections=arguments.logging_sections,
-            log_identifier=arguments.log_identifier,
-            logger=arguments.logger,
-            formatter=arguments.formatter,
-            targets=arguments.targets,
-            use_buck_builder=arguments.use_buck_builder,
-            source_directories=arguments.source_directories,
-            filter_directory=arguments.filter_directory,
-            buck_mode=arguments.buck_mode,
-            no_saved_state=arguments.no_saved_state,
-            search_path=arguments.search_path,
-            binary=arguments.binary,
-            buck_builder_binary=arguments.buck_builder_binary,
-            exclude=arguments.exclude,
-            typeshed=arguments.typeshed,
-            save_initial_state_to=arguments.save_initial_state_to,
-            load_initial_state_from=arguments.load_initial_state_from,
-            changed_files_path=arguments.changed_files_path,
-            saved_state_project=arguments.saved_state_project,
-            dot_pyre_directory=arguments.dot_pyre_directory,
-            features=arguments.features,
-        )
-
-
 class CommandParser(ABC):
     NAME = ""
     HIDDEN = False
     _exit_code: ExitCode = ExitCode.SUCCESS
 
-    def __init__(
-        self, command_arguments: CommandArguments, original_directory: str
-    ) -> None:
-        self._command_arguments = command_arguments
-
-        self._version: bool = self._command_arguments.version
-        self._debug: bool = self._command_arguments.debug
-        self._sequential: bool = self._command_arguments.sequential
-        self._strict: bool = self._command_arguments.strict
-        self._additional_checks: List[str] = self._command_arguments.additional_checks
-        self._show_error_traces: bool = self._command_arguments.show_error_traces
-        self._output: str = self._command_arguments.output
-        self._enable_profiling: bool = self._command_arguments.enable_profiling
-        self._enable_memory_profiling: bool = (
-            self._command_arguments.enable_memory_profiling
-        )
-        self._noninteractive: bool = self._command_arguments.noninteractive
-        self._hide_parse_errors: bool = self._command_arguments.hide_parse_errors
-        self._log_identifier: str = self._command_arguments.log_identifier
-        self._formatter: List[str] = self._command_arguments.formatter
-
-        self._targets: List[str] = self._command_arguments.targets
-        self._use_buck_builder: Optional[
-            bool
-        ] = self._command_arguments.use_buck_builder
-
-        self._source_directories: List[str] = self._command_arguments.source_directories
-        self._filter_directory: Optional[str] = self._command_arguments.filter_directory
-        self._buck_mode: Optional[str] = self._command_arguments.buck_mode
-        self._no_saved_state: bool = self._command_arguments.no_saved_state
-
-        self._search_path: List[str] = self._command_arguments.search_path
-        self._binary: str = self._command_arguments.binary
-        self._buck_builder_binary: Final[
-            Optional[str]
-        ] = self._command_arguments.buck_builder_binary
-        self._exclude: List[str] = self._command_arguments.exclude
-        self._typeshed: str = self._command_arguments.typeshed
-        self._save_initial_state_to: Final[
-            Optional[str]
-        ] = self._command_arguments.save_initial_state_to
-        self._load_initial_state_from: Final[
-            Optional[str]
-        ] = self._command_arguments.load_initial_state_from
-        self._changed_files_path: Final[
-            Optional[str]
-        ] = self._command_arguments.changed_files_path
-        self._saved_state_project: Final[
-            Optional[str]
-        ] = self._command_arguments.saved_state_project
-
-        # Derived arguments
-        self._capable_terminal: bool = terminal.is_capable()
-        self._original_directory: str = original_directory
-        self._project_root: str = find_project_root(self._original_directory)
-
-        local_root = self._command_arguments.local_configuration
-        if local_root and local_root.endswith(LOCAL_CONFIGURATION_FILE):
-            local_root = local_root[: -len(LOCAL_CONFIGURATION_FILE)]
-        self._local_root: Final[Optional[str]] = find_local_root(
-            self._original_directory, local_root
-        )
-        self._dot_pyre_directory: Path = (
-            self._command_arguments.dot_pyre_directory
-            or Path(self._project_root, LOG_DIRECTORY)
-        )
-        self._log_directory: str = find_log_directory(
-            self._project_root, self._local_root, str(self._dot_pyre_directory)
-        )
-        Path(self._log_directory).mkdir(parents=True, exist_ok=True)
-
-        self._logging_sections: Optional[str] = self._command_arguments.logging_sections
-        self._noninteractive: bool = self._command_arguments.noninteractive
-        if self._debug or not self._capable_terminal:
-            self._noninteractive = True
+    def __init__(self) -> None:
+        pass
 
     @classmethod
     def add_arguments(cls, parser: argparse.ArgumentParser) -> None:
@@ -391,6 +234,12 @@ class CommandParser(ABC):
         buck_arguments.add_argument(
             "--buck-mode", type=str, help="Mode to pass to `buck query`"
         )
+        buck_arguments.add_argument(
+            "--use-buck-source-database",
+            action="store_const",
+            const=True,
+            help=argparse.SUPPRESS,
+        )
 
         source_directories = parser.add_argument_group("source-directories")
         source_directories.add_argument(
@@ -487,125 +336,73 @@ class CommandParser(ABC):
         self._run()
         return self
 
-    def cleanup(self) -> None:
-        pass
-
     def exit_code(self) -> ExitCode:
         return self._exit_code
 
-    def result(self) -> Optional[Result]:
-        return None
-
     @property
     def configuration(self) -> Optional[Configuration]:
         return None
 
-    @property
-    def project_root(self) -> Optional[str]:
-        return self._project_root
-
-    @property
-    def local_root(self) -> Optional[str]:
-        return self._local_root
-
-    @property
-    def log_directory(self) -> str:
-        return self._log_directory
-
-    @property
-    def noninteractive(self) -> bool:
-        return self._noninteractive
-
-    @property
-    def relative_local_root(self) -> Optional[str]:
-        if not self.local_root:
-            return None
-        return str(Path(self.log_directory).relative_to(self._dot_pyre_directory))
-
 
 class Command(CommandParser, ABC):
     _buffer: List[str] = []
 
     def __init__(
         self,
-        command_arguments: CommandArguments,
+        command_arguments: command_arguments.CommandArguments,
         original_directory: str,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
     ) -> None:
-        super(Command, self).__init__(command_arguments, original_directory)
-        logger = self._command_arguments.logger
-        if logger:
-            logger = translate_path(self._original_directory, logger)
-        self._configuration: Configuration = (
-            configuration or self.generate_configuration(logger)
-        )
-        self._strict: bool = (
-            self._command_arguments.strict or self._configuration.strict
-        )
-        self._logger: Final[Optional[str]] = logger or (self._configuration.logger)
-        self._ignore_all_errors_paths: Iterable[str] = (
-            self._configuration.ignore_all_errors
-        )
-        self._number_of_workers: int = self._configuration.number_of_workers
+        super().__init__()
+        self._command_arguments = command_arguments
+
+        self._show_error_traces: bool = self._command_arguments.show_error_traces
+        self._output: str = self._command_arguments.output
+
+        # Derived arguments
+        self._capable_terminal: bool = terminal.is_capable()
+        self._original_directory: str = original_directory
+
+        self._logging_sections: Optional[str] = self._command_arguments.logging_sections
+        self._noninteractive: bool = self._command_arguments.noninteractive
+        if self._command_arguments.debug or not self._capable_terminal:
+            self._noninteractive = True
+
+        self._configuration: Configuration = configuration
         self._version_hash: str = self._configuration.version_hash
-        self._formatter: Final[Optional[str]] = self._configuration.formatter
-        self._taint_models_path: List[str] = [
-            translate_path(self._original_directory, path)
-            for path in self._configuration.taint_models_path
-        ]
+        self._taint_models_path: List[str] = self._configuration.taint_models_path
 
         self._analysis_directory: AnalysisDirectory = (
             analysis_directory or self.generate_analysis_directory()
         )
-        self._features: Final[Optional[str]] = self._command_arguments.features
 
     @classmethod
     def add_subparser(cls, parser: argparse._SubParsersAction) -> None:
         pass
 
-    def generate_configuration(self, logger: Optional[str]) -> Configuration:
-        return Configuration(
-            project_root=self._project_root,
-            local_root=self._local_root,
-            search_path=self._search_path,
-            binary=self._binary,
-            typeshed=self._typeshed,
-            buck_builder_binary=self._buck_builder_binary,
-            excludes=self._exclude,
-            logger=logger,
-            formatter=self._formatter,
-            log_directory=self._log_directory,
+    def generate_analysis_directory(self) -> AnalysisDirectory:
+        return resolve_analysis_directory(
+            self._command_arguments.source_directories,
+            self._command_arguments.targets,
+            self._configuration,
+            self._original_directory,
+            self._configuration.project_root,
+            filter_directory=self._command_arguments.filter_directory,
+            buck_mode=self._command_arguments.buck_mode,
+            relative_local_root=self._configuration.relative_local_root,
         )
 
-    def generate_analysis_directory(self) -> AnalysisDirectory:
-        configuration = self._configuration
-        if not configuration:
-            return AnalysisDirectory(".")
-        else:
-            return resolve_analysis_directory(
-                self._source_directories,
-                self._targets,
-                configuration,
-                self._original_directory,
-                self._project_root,
-                filter_directory=self._filter_directory,
-                use_buck_builder=self._use_buck_builder,
-                debug=self._debug,
-                buck_mode=self._buck_mode,
-                relative_local_root=self.relative_local_root,
-            )
-
     def run(self) -> "Command":
-        configuration = self._configuration
-        if configuration and configuration.disabled:
+        if self._configuration.disabled:
             LOG.log(log.SUCCESS, "Pyre will not run due to being explicitly disabled")
         else:
-            relative_local_root = self.relative_local_root
+            relative_local_root = self._configuration.relative_local_root
             if relative_local_root:
-                recently_used_configurations.Cache(self._dot_pyre_directory).put(
-                    relative_local_root
-                )
+                recently_used_configurations.Cache(
+                    self._configuration.dot_pyre_directory
+                ).put(relative_local_root)
+            Path(self._configuration.log_directory).mkdir(parents=True, exist_ok=True)
             self._run()
         return self
 
@@ -617,19 +414,18 @@ class Command(CommandParser, ABC):
 
     def _flags(self) -> List[str]:
         flags = []
-        if self._debug:
+        if self._command_arguments.debug:
             flags.extend(["-debug"])
-        if self._sequential:
+        if self._command_arguments.sequential:
             flags.extend(["-sequential"])
-        if self._strict:
+        if self._configuration.strict:
             flags.extend(["-strict"])
-        if self._additional_checks:
+        additional_checks = self._command_arguments.additional_checks
+        if additional_checks:
             flags.append("-additional-checks")
-            flags.append(",".join(self._additional_checks))
+            flags.append(",".join(additional_checks))
         if self._show_error_traces:
             flags.append("-show-error-traces")
-        if not self._hide_parse_errors:
-            self._enable_logging_section("parser")
         logging_sections = self._logging_sections
         if not self._capable_terminal:
             # Disable progress reporting for non-capable terminals.
@@ -640,26 +436,29 @@ class Command(CommandParser, ABC):
                 logging_sections = "-progress"
         if logging_sections:
             flags.extend(["-logging-sections", logging_sections])
-        if self._enable_profiling:
+        if self._command_arguments.enable_profiling:
             flags.extend(["-profiling-output", self.profiling_log_path()])
-        if self._enable_memory_profiling:
+        if self._command_arguments.enable_memory_profiling:
             flags.extend(["-memory-profiling-output", self.profiling_log_path()])
-        if self._enable_profiling or self._enable_memory_profiling:
+        if (
+            self._command_arguments.enable_profiling
+            or self._command_arguments.enable_memory_profiling
+        ):
             # Clear the profiling log first since in pyre binary it's append-only
             remove_if_exists(self.profiling_log_path())
-        if self._project_root:
-            flags.extend(["-project-root", self._project_root])
-        if self._log_identifier:
-            flags.extend(["-log-identifier", self._log_identifier])
-        if self._logger:
-            flags.extend(["-logger", self._logger])
-        if self._log_directory:
-            flags.extend(["-log-directory", self._log_directory])
+        flags.extend(["-project-root", self._configuration.project_root])
+        log_identifier = self._command_arguments.log_identifier
+        if log_identifier:
+            flags.extend(["-log-identifier", log_identifier])
+        logger = self._configuration.logger
+        if logger:
+            flags.extend(["-logger", logger])
+        flags.extend(["-log-directory", self._configuration.log_directory])
         return flags
 
     # temporarily always return empty list to unblock client release
     def _feature_flags(self) -> List[str]:
-        features = self._features
+        features = self._command_arguments.features
         if features:
             lsp_features = ["click_to_fix", "hover", "go_to_definition"]
             filtered = {
@@ -724,7 +523,7 @@ class Command(CommandParser, ABC):
             #  `Optional[IO[typing.Any]]`.
             with StreamLogger(process.stderr) as stream_logger:
                 with Process.register_non_unique_process(
-                    process.pid, self.NAME, self.log_directory
+                    process.pid, self.NAME, self._configuration.log_directory
                 ):
                     # Wait for the process to finish and clean up.
                     process.wait()
@@ -747,7 +546,7 @@ class Command(CommandParser, ABC):
         return os.path.relpath(path, self._original_directory)
 
     def _state(self) -> State:
-        pid_path = Path(self._log_directory, "server/server.pid")
+        pid_path = Path(self._configuration.log_directory, "server/server.pid")
         return State.RUNNING if Process.is_alive(pid_path) else State.DEAD
 
     # will open a socket, send a request, read the response and close the socket.
@@ -755,7 +554,9 @@ class Command(CommandParser, ABC):
         self, request: json_rpc.Request, version_hash: str
     ) -> None:
         try:
-            stderr_file = os.path.join(self._log_directory, "server/server.stdout")
+            stderr_file = os.path.join(
+                self._configuration.log_directory, "server/server.stdout"
+            )
             with subprocess.Popen(
                 ["tail", "--follow", "--lines=0", stderr_file],
                 stdout=subprocess.PIPE,
@@ -763,7 +564,9 @@ class Command(CommandParser, ABC):
                 universal_newlines=True,
             ) as stderr_tail:
                 try:
-                    with SocketConnection(self._log_directory) as socket_connection:
+                    with SocketConnection(
+                        self._configuration.log_directory
+                    ) as socket_connection:
                         socket_connection.perform_handshake(version_hash)
                         # pyre-fixme[6]: Expected `Iterable[str]` for 1st param but
                         #  got `Optional[IO[typing.Any]]`.
@@ -791,7 +594,7 @@ class Command(CommandParser, ABC):
         log.stdout.write(result.output)
 
     def profiling_log_path(self) -> str:
-        return os.path.join(self._log_directory, "profiling.log")
+        return os.path.join(self._configuration.log_directory, "profiling.log")
 
     @property
     def analysis_directory(self) -> AnalysisDirectory:
@@ -807,3 +610,10 @@ class Command(CommandParser, ABC):
             self._logging_sections = logging_sections + "," + section
         else:
             self._logging_sections = section
+
+    def result(self) -> Optional[Result]:
+        return None
+
+    @property
+    def noninteractive(self) -> bool:
+        return self._noninteractive
