@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,18 +8,10 @@ import logging
 from logging import Logger
 from typing import List, Optional
 
-from .. import json_rpc
+from .. import command_arguments, configuration_monitor, json_rpc, project_files_monitor
 from ..analysis_directory import AnalysisDirectory
 from ..configuration import Configuration
-from ..project_files_monitor import ProjectFilesMonitor
-from .command import (
-    CommandArguments,
-    ExitCode,
-    IncrementalStyle,
-    Result,
-    State,
-    typeshed_search_path,
-)
+from .command import ExitCode, IncrementalStyle, Result, State, typeshed_search_path
 from .reporting import Reporting
 from .start import Start
 
@@ -32,10 +24,10 @@ class Incremental(Reporting):
 
     def __init__(
         self,
-        command_arguments: CommandArguments,
+        command_arguments: command_arguments.CommandArguments,
         original_directory: str,
         *,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
         nonblocking: bool,
         incremental_style: IncrementalStyle,
@@ -54,11 +46,11 @@ class Incremental(Reporting):
     def from_arguments(
         arguments: argparse.Namespace,
         original_directory: str,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
     ) -> "Incremental":
         return Incremental(
-            CommandArguments.from_arguments(arguments),
+            command_arguments.CommandArguments.from_arguments(arguments),
             original_directory,
             configuration=configuration,
             analysis_directory=analysis_directory,
@@ -105,27 +97,44 @@ class Incremental(Reporting):
         )
 
     def _run(self) -> None:
-        if (not self._no_start_server) and self._state() == State.DEAD:
-            LOG.info("Starting server at `%s`.", self._analysis_directory.get_root())
-            exit_code = (
-                Start(
-                    self._command_arguments,
-                    self._original_directory,
-                    configuration=self._configuration,
-                    analysis_directory=self._analysis_directory,
-                    terminal=False,
-                    store_type_check_resolution=False,
-                    use_watchman=not self._no_watchman,
-                    incremental_style=self._incremental_style,
+        if self._state() == State.DEAD:
+            if not self._no_start_server:
+                LOG.info(
+                    "Starting server at `%s`.", self._analysis_directory.get_root()
                 )
-                .run()
-                .exit_code()
-            )
-            if exit_code != ExitCode.SUCCESS:
-                self._exit_code = ExitCode.FAILURE
-                return
+                exit_code = (
+                    Start(
+                        self._command_arguments,
+                        self._original_directory,
+                        configuration=self._configuration,
+                        analysis_directory=self._analysis_directory,
+                        terminal=False,
+                        store_type_check_resolution=False,
+                        use_watchman=not self._no_watchman,
+                        incremental_style=self._incremental_style,
+                    )
+                    .run()
+                    .exit_code()
+                )
+                if exit_code != ExitCode.SUCCESS:
+                    self._exit_code = ExitCode.FAILURE
+                    return
         else:
-            self._restart_file_monitor_if_needed()
+            if not self._no_watchman and (
+                not project_files_monitor.ProjectFilesMonitor.is_alive(
+                    self._configuration
+                )
+                or not configuration_monitor.ConfigurationMonitor.is_alive(
+                    self._configuration
+                )
+            ):
+                LOG.warning(
+                    "File watching service is down. Results may be inconsistent with "
+                    "full checks. Please run `pyre restart` to bring Pyre server to a "
+                    "consistent state again."
+                )
+                self._exit_code = ExitCode.INCONSISTENT_SERVER
+                return
 
         if self._state() != State.DEAD:
             LOG.info("Waiting for server...")
@@ -164,10 +173,3 @@ class Incremental(Reporting):
             flags.append("-nonblocking")
 
         return flags
-
-    def _restart_file_monitor_if_needed(self) -> None:
-        if self._no_watchman:
-            return
-        ProjectFilesMonitor.restart_if_dead(
-            self._configuration, self._project_root, self._analysis_directory
-        )
