@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -19,16 +19,15 @@ from pathlib import Path
 from typing import IO, Any, List, Optional, Sequence, Set, Union
 
 import libcst
-from libcst._version import LIBCST_VERSION
 from libcst.codemod import CodemodContext
 from libcst.codemod.visitors._apply_type_annotations import ApplyTypeAnnotationsVisitor
 from typing_extensions import Final
 
-from .. import log
+from .. import command_arguments, log
 from ..analysis_directory import AnalysisDirectory
 from ..configuration import Configuration
 from ..error import Error
-from .command import JSON, Command, CommandArguments, Result, typeshed_search_path
+from .command import JSON, Command, Result, typeshed_search_path
 from .reporting import Reporting
 
 
@@ -273,7 +272,7 @@ class StubFile:
 
         for stub in stubs_in_file:
             typing_imports.update(stub.get_typing_imports())
-            alphabetical_imports = sorted(list(typing_imports))
+            alphabetical_imports = sorted(typing_imports)
 
         if alphabetical_imports and contents != "":
             contents = (
@@ -430,10 +429,10 @@ class Infer(Reporting):
 
     def __init__(
         self,
-        command_arguments: CommandArguments,
+        command_arguments: command_arguments.CommandArguments,
         original_directory: str,
         *,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
         print_errors: bool,
         full_only: bool,
@@ -462,11 +461,11 @@ class Infer(Reporting):
     def from_arguments(
         arguments: argparse.Namespace,
         original_directory: str,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
     ) -> "Infer":
         return Infer(
-            CommandArguments.from_arguments(arguments),
+            command_arguments.CommandArguments.from_arguments(arguments),
             original_directory,
             configuration=configuration,
             analysis_directory=analysis_directory,
@@ -540,10 +539,12 @@ class Infer(Reporting):
                     --in-place argument"
                 )
 
-            type_directory = Path(os.path.join(self._log_directory, "types"))
+            type_directory = Path(
+                os.path.join(self._configuration.log_directory, "types")
+            )
             annotate_from_existing_stubs(
                 Path(self._original_directory),
-                self._formatter,
+                self._configuration.formatter,
                 type_directory,
                 self._in_place,
                 self._debug_infer,
@@ -557,14 +558,16 @@ class Infer(Reporting):
         if self._print_errors:
             self._print(errors)
         else:
-            type_directory = Path(os.path.join(self._log_directory, "types"))
+            type_directory = Path(
+                os.path.join(self._configuration.log_directory, "types")
+            )
             stubs = generate_stub_files(self._full_only, errors)
             write_stubs_to_disk(stubs, type_directory)
             if self._in_place is not None:
                 LOG.info("Annotating files")
                 annotate_paths(
                     self._configuration.local_root,
-                    self._formatter,
+                    self._configuration.formatter,
                     stubs,
                     type_directory,
                     self._in_place,
@@ -577,8 +580,6 @@ class Infer(Reporting):
         flags = super()._flags()
         filter_directories = self._get_directories_to_analyze()
         if len(filter_directories):
-            # pyre-fixme[6]: Expected `Iterable[Variable[_LT (bound to
-            #  _SupportsLessThan)]]` for 1st param but got `Set[str]`.
             flags.extend(["-filter-directories", ";".join(sorted(filter_directories))])
         search_path = self._configuration.search_path + typeshed_search_path(
             self._configuration.typeshed
