@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -12,10 +12,10 @@ import re
 from logging import Logger
 from typing import Dict, List, Optional
 
-from .. import json_rpc, log
+from .. import command_arguments, json_rpc, log
 from ..analysis_directory import AnalysisDirectory
 from ..configuration import Configuration
-from .command import Command, CommandArguments, Result
+from .command import Command, ExitCode, Result, State
 
 
 LOG: Logger = logging.getLogger(__name__)
@@ -47,10 +47,10 @@ class Query(Command):
 
     def __init__(
         self,
-        command_arguments: CommandArguments,
+        command_arguments: command_arguments.CommandArguments,
         original_directory: str,
         *,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
         query: str,
     ) -> None:
@@ -63,11 +63,11 @@ class Query(Command):
     def from_arguments(
         arguments: argparse.Namespace,
         original_directory: str,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
     ) -> "Query":
         return Query(
-            CommandArguments.from_arguments(arguments),
+            command_arguments.CommandArguments.from_arguments(arguments),
             original_directory,
             configuration=configuration,
             analysis_directory=analysis_directory,
@@ -94,12 +94,14 @@ class Query(Command):
 
     def _flags(self) -> List[str]:
         flags = [self.query]
-        log_directory = self._log_directory
-        if log_directory:
-            flags.extend(["-log-directory", log_directory])
+        flags.extend(["-log-directory", self._configuration.log_directory])
         return flags
 
     def _run(self) -> None:
+        if self._state() == State.DEAD:
+            LOG.error("No server running to query.")
+            self._exit_code = ExitCode.SERVER_NOT_FOUND
+            return
         LOG.info("Waiting for server...")
         with self._analysis_directory.acquire_shared_reader_lock():
             request = json_rpc.Request(
