@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,12 +10,15 @@ import os
 from logging import Logger
 from typing import List, Optional
 
-from typing_extensions import Final
-
-from .. import configuration_monitor, filesystem, project_files_monitor
+from .. import (
+    command_arguments,
+    configuration_monitor,
+    filesystem,
+    project_files_monitor,
+)
 from ..analysis_directory import AnalysisDirectory
 from ..configuration import Configuration
-from .command import CommandArguments, IncrementalStyle, typeshed_search_path
+from .command import IncrementalStyle, typeshed_search_path
 from .reporting import Reporting
 
 
@@ -27,10 +30,10 @@ class Start(Reporting):
 
     def __init__(
         self,
-        command_arguments: CommandArguments,
+        command_arguments: command_arguments.CommandArguments,
         original_directory: str,
         *,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
         terminal: bool,
         store_type_check_resolution: bool,
@@ -44,11 +47,6 @@ class Start(Reporting):
         self._store_type_check_resolution = store_type_check_resolution
         self._use_watchman = use_watchman
         self._incremental_style = incremental_style
-        if self._no_saved_state:
-            self._save_initial_state_to: Final[Optional[str]] = None
-            self._changed_files_path: Final[Optional[str]] = None
-            self._load_initial_state_from: Final[Optional[str]] = None
-            self._saved_state_project: Final[Optional[str]] = None
 
         self._enable_logging_section("environment")
 
@@ -56,11 +54,11 @@ class Start(Reporting):
     def from_arguments(
         arguments: argparse.Namespace,
         original_directory: str,
-        configuration: Optional[Configuration] = None,
+        configuration: Configuration,
         analysis_directory: Optional[AnalysisDirectory] = None,
     ) -> "Start":
         return Start(
-            CommandArguments.from_arguments(arguments),
+            command_arguments.CommandArguments.from_arguments(arguments),
             original_directory,
             configuration=configuration,
             analysis_directory=analysis_directory,
@@ -101,15 +99,15 @@ class Start(Reporting):
                 self._command_arguments,
                 self._configuration,
                 self._analysis_directory,
-                self._project_root,
+                self._configuration.project_root,
                 self._original_directory,
-                self.local_root,
+                self._configuration.local_root,
                 self._configuration.other_critical_files,
             ).daemonize()
 
     def _run(self) -> None:
         blocking = False
-        lock = os.path.join(self._log_directory, "client.lock")
+        lock = os.path.join(self._configuration.log_directory, "client.lock")
         while True:
             # Be optimistic in grabbing the lock in order to provide users with
             # a message when the lock is being waited on.
@@ -121,7 +119,11 @@ class Start(Reporting):
                     # spawn in the interim which would cause a race.
                     try:
                         with filesystem.acquire_lock(
-                            os.path.join(self._log_directory, "server", "server.lock"),
+                            os.path.join(
+                                self._configuration.log_directory,
+                                "server",
+                                "server.lock",
+                            ),
                             blocking=False,
                         ):
                             pass
@@ -140,7 +142,7 @@ class Start(Reporting):
                         try:
                             file_monitor = project_files_monitor.ProjectFilesMonitor(
                                 self._configuration,
-                                self._project_root,
+                                self._configuration.project_root,
                                 self._analysis_directory,
                             )
                             file_monitor.daemonize()
@@ -163,8 +165,6 @@ class Start(Reporting):
                 flags.extend(["-taint-models", path])
         filter_directories = self._get_directories_to_analyze()
         if len(filter_directories):
-            # pyre-fixme[6]: Expected `Iterable[Variable[_LT (bound to
-            #  _SupportsLessThan)]]` for 1st param but got `Set[str]`.
             flags.extend(["-filter-directories", ";".join(sorted(filter_directories))])
         if len(self._configuration.ignore_all_errors):
             flags.extend(
@@ -177,35 +177,37 @@ class Start(Reporting):
             flags.append("-terminal")
         if self._store_type_check_resolution:
             flags.append("-store-type-check-resolution")
-        save_initial_state_to = self._save_initial_state_to
-        if save_initial_state_to and os.path.isdir(
-            os.path.dirname(save_initial_state_to)
-        ):
-            flags.extend(["-save-initial-state-to", save_initial_state_to])
-        saved_state_project = self._saved_state_project
-        if saved_state_project:
-            flags.extend(["-saved-state-project", saved_state_project])
-            local_root = self._configuration.local_root
-            if local_root is not None:
-                relative = os.path.relpath(local_root)
-                flags.extend(["-saved-state-metadata", relative.replace("/", "$")])
-        configuration_file_hash = self._configuration.file_hash
-        if configuration_file_hash:
-            flags.extend(["-configuration-file-hash", configuration_file_hash])
-        load_initial_state_from = self._load_initial_state_from
-        if load_initial_state_from is not None:
-            flags.extend(["-load-state-from", load_initial_state_from])
-            changed_files_path = self._changed_files_path
-            if changed_files_path is not None:
-                flags.extend(["-changed-files-path", changed_files_path])
-        elif self._changed_files_path is not None:
-            LOG.error(
-                "--load-initial-state-from must be set if --changed-files-path is set."
-            )
+        if not self._command_arguments.no_saved_state:
+            save_initial_state_to = self._command_arguments.save_initial_state_to
+            if save_initial_state_to and os.path.isdir(
+                os.path.dirname(save_initial_state_to)
+            ):
+                flags.extend(["-save-initial-state-to", save_initial_state_to])
+            saved_state_project = self._command_arguments.saved_state_project
+            if saved_state_project:
+                flags.extend(["-saved-state-project", saved_state_project])
+                relative_local_root = self._configuration.relative_local_root
+                if relative_local_root is not None:
+                    flags.extend(
+                        ["-saved-state-metadata", relative_local_root.replace("/", "$")]
+                    )
+            configuration_file_hash = self._configuration.file_hash
+            if configuration_file_hash:
+                flags.extend(["-configuration-file-hash", configuration_file_hash])
+            load_initial_state_from = self._command_arguments.load_initial_state_from
+            changed_files_path = self._command_arguments.changed_files_path
+            if load_initial_state_from is not None:
+                flags.extend(["-load-state-from", load_initial_state_from])
+                if changed_files_path is not None:
+                    flags.extend(["-changed-files-path", changed_files_path])
+            elif changed_files_path is not None:
+                LOG.error(
+                    "--load-initial-state-from must be set if --changed-files-path is set."
+                )
         flags.extend(
             [
                 "-workers",
-                str(self._number_of_workers),
+                str(self._configuration.number_of_workers),
                 "-expected-binary-version",
                 self._configuration.version_hash,
             ]
