@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -19,55 +19,74 @@ from logging import Logger
 from pathlib import Path
 from typing import Dict, List, Optional, Union
 
+from . import command_arguments, find_directories
 from .exceptions import EnvironmentException
-from .filesystem import assert_readable_directory, expand_relative_path, find_root
+from .filesystem import assert_readable_directory, expand_relative_path
 from .find_directories import (
     BINARY_NAME,
     CONFIGURATION_FILE,
     LOCAL_CONFIGURATION_FILE,
+    find_parent_directory_containing_file,
     find_typeshed,
+    get_relative_local_root,
 )
+from .resources import LOG_DIRECTORY
 
 
 LOG: Logger = logging.getLogger(__name__)
 
 
 class InvalidConfiguration(Exception):
-    pass
+    def __init__(self, message: str) -> None:
+        self.message = f"Invalid configuration: {message}"
+        super().__init__(self.message)
 
 
 class SearchPathElement:
-    def __init__(self, root: str, subdirectory: Optional[str] = None) -> None:
-        self.root = os.path.expanduser(root)
+    def __init__(self, root: str, subdirectory: Optional[str]) -> None:
+        self.root = root
         self.subdirectory = subdirectory
 
     @staticmethod
     def expand(
-        path: Union[Dict[str, str], str], base: Optional[str] = None
+        path: Union[Dict[str, str], str],
+        project_root: str,
+        path_relative_to: Optional[str] = None,
     ) -> "SearchPathElement":
         if isinstance(path, str):
-            if base:
-                path = expand_relative_path(base, path)
-            return SearchPathElement(path)
+            return SearchPathElement(
+                root=SearchPathElement.relativize_root(
+                    path, project_root, path_relative_to
+                ),
+                subdirectory=None,
+            )
         else:
             if "root" in path and "subdirectory" in path:
                 root = path["root"]
                 subdirectory = path["subdirectory"]
-                if base:
-                    root = expand_relative_path(base, root)
-                return SearchPathElement(root, subdirectory)
+                return SearchPathElement(
+                    root=SearchPathElement.relativize_root(
+                        root, project_root, path_relative_to
+                    ),
+                    subdirectory=subdirectory,
+                )
             elif "site-package" in path:
                 site_root = site.getsitepackages()
                 subdirectory = path["site-package"]
 
                 found_element = None
                 for root in site_root:
-                    site_package_element = SearchPathElement(root, subdirectory)
+                    site_package_element = SearchPathElement(
+                        root=SearchPathElement.relativize_root(
+                            root, project_root, None
+                        ),
+                        subdirectory=subdirectory,
+                    )
                     if os.path.isdir(site_package_element.path()):
                         found_element = site_package_element
                 if found_element is None:
                     raise InvalidConfiguration(
-                        "Cannot find site package '{}'".format(subdirectory)
+                        f"Cannot find site package '{subdirectory}'"
                     )
                 return found_element
             else:
@@ -76,6 +95,15 @@ class SearchPathElement:
                     "specified."
                 )
 
+    @staticmethod
+    def relativize_root(
+        root: str, project_root: str, relative_root: Optional[str]
+    ) -> str:
+        if root.startswith("//"):
+            return expand_relative_path(project_root, root[2:])
+        else:
+            return expand_relative_path(relative_root or "", root)
+
     def path(self) -> str:
         subdirectory = self.subdirectory
         if subdirectory is not None:
@@ -98,6 +126,15 @@ class SearchPathElement:
             return self.root == other.root and self.subdirectory == other.subdirectory
 
 
+def assert_readable_directory_in_configuration(
+    directory: str, field_name: str = ""
+) -> None:
+    try:
+        assert_readable_directory(directory, error_message_prefix=f"{field_name} ")
+    except EnvironmentException as error:
+        raise InvalidConfiguration(str(error))
+
+
 class _ConfigurationFile:
     def __init__(self, file) -> None:
         self._deprecated = {"do_not_check": "ignore_all_errors"}
@@ -125,7 +162,7 @@ class _ConfigurationFile:
         value = self._configuration.pop(key, default)
         if raise_on_override and current and value:
             raise EnvironmentException(
-                "Configuration file may not override `{}` field.".format(key)
+                f"Configuration file may not override `{key}` field."
             )
         if current:
             return current
@@ -160,15 +197,18 @@ class Configuration:
     def __init__(
         self,
         project_root: str,
+        dot_pyre_directory: Path,
         local_root: Optional[str] = None,
         search_path: Optional[List[str]] = None,
         binary: Optional[str] = None,
         typeshed: Optional[str] = None,
+        use_buck_builder: Optional[bool] = None,
         buck_builder_binary: Optional[str] = None,
         excludes: Optional[List[str]] = None,
+        strict: bool = False,
         formatter: Optional[str] = None,
         logger: Optional[str] = None,
-        log_directory: Optional[str] = None,
+        use_buck_source_database: Optional[bool] = None,
     ) -> None:
         self.source_directories = []
         self.targets = []
@@ -181,20 +221,32 @@ class Configuration:
         self.taint_models_path: List[str] = []
         self.file_hash: Optional[str] = None
         self.extensions: List[str] = []
-        self._log_directory: Optional[str] = log_directory
+        self.dot_pyre_directory: Path = dot_pyre_directory
+
+        relative_local_root = get_relative_local_root(
+            Path(project_root), Path(local_root) if local_root is not None else None
+        )
+        self.relative_local_root: Optional[str] = relative_local_root
+        self.log_directory: str = str(
+            self.dot_pyre_directory
+            if relative_local_root is None
+            else self.dot_pyre_directory / relative_local_root
+        )
 
         self._version_hash: Optional[str] = None
         self._binary: Optional[str] = None
         self._typeshed: Optional[str] = None
         self._buck_builder_binary: Optional[str] = None
-        self.strict: bool = False
-        self._use_buck_builder: Optional[bool] = None
+        self.strict: bool = strict
+        self._use_buck_builder: Optional[bool] = use_buck_builder
+        self._use_buck_source_database: Optional[bool] = use_buck_source_database
         self.ignore_infer: List[str] = []
 
         self._search_path: List[SearchPathElement] = []
         if search_path:
             search_path_elements = [
-                SearchPathElement.expand(path) for path in search_path
+                SearchPathElement.expand(path, project_root=project_root)
+                for path in search_path
             ]
             self._search_path.extend(search_path_elements)
         # We will extend the search path further, with the config file
@@ -215,8 +267,7 @@ class Configuration:
 
         if local_root:
             self._check_read_local_configuration(local_root)
-        if log_directory:
-            self.ignore_all_errors.append(log_directory)
+        self.ignore_all_errors.append(os.path.abspath(self.log_directory))
 
         self.autocomplete = False
 
@@ -231,6 +282,39 @@ class Configuration:
         self._apply_defaults()
         self._validate()
 
+    @staticmethod
+    def from_arguments(
+        arguments: command_arguments.CommandArguments, base_directory: Path
+    ) -> "Optional[Configuration]":
+        local_root_argument = arguments.local_configuration
+        found_root = find_directories.find_global_and_local_root(
+            base_directory
+            if local_root_argument is None
+            else base_directory / local_root_argument
+        )
+        if found_root is None:
+            return None
+
+        global_root = found_root.global_root
+        local_root = found_root.local_root
+        return Configuration(
+            project_root=str(global_root),
+            local_root=str(local_root) if local_root is not None else None,
+            search_path=arguments.search_path,
+            binary=arguments.binary,
+            typeshed=arguments.typeshed,
+            use_buck_builder=arguments.use_buck_builder,
+            buck_builder_binary=arguments.buck_builder_binary,
+            excludes=arguments.exclude,
+            strict=arguments.strict,
+            logger=arguments.logger,
+            formatter=arguments.formatter,
+            dot_pyre_directory=(
+                arguments.dot_pyre_directory or global_root / LOG_DIRECTORY
+            ),
+            use_buck_source_database=arguments.use_buck_source_database,
+        )
+
     def _validate(self) -> None:
         try:
 
@@ -272,22 +356,22 @@ class Configuration:
                 )
 
             if not os.path.exists(self.binary):
-                raise InvalidConfiguration(
-                    "Binary at `{}` does not exist.".format(self.binary)
-                )
+                raise InvalidConfiguration(f"Binary at `{self.binary}` does not exist.")
 
             if self.number_of_workers < 1:
                 raise InvalidConfiguration("Number of workers must be greater than 0.")
 
             # Validate typeshed path and sub-elements.
-            assert_readable_directory(self.typeshed)
+            assert_readable_directory_in_configuration(
+                self.typeshed, field_name="typeshed"
+            )
 
             # A courtesy warning since we have changed default behaviour.
             if self._typeshed_has_obsolete_value():
                 LOG.warning(
-                    "It appears that `{}` points at a `stdlib` directory. "
-                    "Please note that the `typeshed` configuration must point at "
-                    "the root of the `typeshed` directory.".format(self.typeshed)
+                    f"It appears that `{self.typeshed}` points at a `stdlib` "
+                    "directory. Please note that the `typeshed` configuration must "
+                    "point to the root of the `typeshed` directory."
                 )
 
             expanded_ignore_paths = []
@@ -304,9 +388,8 @@ class Configuration:
             ]
             if non_existent_ignore_paths:
                 LOG.warning(
-                    "Nonexistent paths passed in to `ignore_all_errors`: `{}`".format(
-                        non_existent_ignore_paths
-                    )
+                    "Nonexistent paths passed in to `ignore_all_errors`: "
+                    f"`{non_existent_ignore_paths}`"
                 )
                 self.ignore_all_errors = [
                     path
@@ -319,9 +402,8 @@ class Configuration:
             ]
             if non_existent_infer_paths:
                 LOG.warning(
-                    "Nonexistent paths passed in to `ignore_infer`: `{}`".format(
-                        non_existent_infer_paths
-                    )
+                    "Nonexistent paths passed in to `ignore_infer`: "
+                    f"`{non_existent_infer_paths}`"
                 )
                 self.ignore_infer = [
                     path
@@ -333,7 +415,7 @@ class Configuration:
                 typeshed_subdirectory = os.path.join(
                     self.typeshed, typeshed_subdirectory_name
                 )
-                assert_readable_directory(typeshed_subdirectory)
+                assert_readable_directory_in_configuration(typeshed_subdirectory)
                 for typeshed_version_directory_name in os.listdir(
                     typeshed_subdirectory
                 ):
@@ -346,13 +428,14 @@ class Configuration:
                     typeshed_version_directory = os.path.join(
                         typeshed_subdirectory, typeshed_version_directory_name
                     )
-                    assert_readable_directory(typeshed_version_directory)
+                    assert_readable_directory_in_configuration(
+                        typeshed_version_directory
+                    )
 
             # Validate elements of the search path.
             for element in self._search_path:
-                assert_readable_directory(
-                    element.path(),
-                    error_message_prefix="Invalid search_path in configuration: ",
+                assert_readable_directory_in_configuration(
+                    element.path(), field_name="search_path"
                 )
 
             if not is_list_of_strings(self.other_critical_files):
@@ -360,7 +443,7 @@ class Configuration:
                     "`critical_files` field must be a list of strings."
                 )
         except InvalidConfiguration as error:
-            raise EnvironmentException("Invalid configuration: {}".format(str(error)))
+            raise EnvironmentException(str(error))
 
     @property
     def version_hash(self) -> str:
@@ -370,14 +453,14 @@ class Configuration:
     def binary(self) -> str:
         binary = self._binary
         if not binary:
-            raise InvalidConfiguration("Configuration was not validated")
+            raise InvalidConfiguration("No binary specified.")
         return binary
 
     @property
     def typeshed(self) -> str:
         typeshed = self._typeshed
         if not typeshed:
-            raise InvalidConfiguration("Configuration invalid: no typeshed specified")
+            raise InvalidConfiguration("No typeshed specified.")
         return typeshed
 
     @property
@@ -388,6 +471,10 @@ class Configuration:
     def use_buck_builder(self) -> bool:
         return self._use_buck_builder or False
 
+    @property
+    def use_buck_source_database(self) -> bool:
+        return self._use_buck_source_database or False
+
     @property
     def search_path(self) -> List[str]:
         if not self._search_path:
@@ -398,13 +485,6 @@ class Configuration:
     def local_root(self) -> Optional[str]:
         return self._local_root
 
-    @property
-    def log_directory(self) -> str:
-        log_directory = self._log_directory
-        if not log_directory:
-            raise InvalidConfiguration("Configuration was not validated")
-        return log_directory
-
     def get_binary_version(self) -> Optional[str]:
         status = subprocess.run(
             [self.binary, "-version"], stdout=subprocess.PIPE, universal_newlines=True
@@ -416,8 +496,8 @@ class Configuration:
 
     def _check_nested_configurations(self, local_root: str) -> None:
         # TODO(T67874463): Handle sanity checks against project configurations
-        parent_local_root = find_root(
-            os.path.dirname(local_root.rstrip("/")), LOCAL_CONFIGURATION_FILE
+        parent_local_root = find_parent_directory_containing_file(
+            Path(local_root.rstrip("/")).parent, LOCAL_CONFIGURATION_FILE
         )
         if not parent_local_root:
             return
@@ -428,7 +508,7 @@ class Configuration:
         try:
             parent_local_configuration = Configuration(
                 project_root=self.project_root,
-                local_root=parent_local_root,
+                local_root=str(parent_local_root),
                 search_path=[search_path.root for search_path in self._search_path],
                 binary=self._binary,
                 typeshed=self._typeshed,
@@ -436,7 +516,7 @@ class Configuration:
                 excludes=self.excludes,
                 logger=self.logger,
                 formatter=self.formatter,
-                log_directory=self._log_directory,
+                dot_pyre_directory=self.dot_pyre_directory,
             )
             paths_to_ignore = list(local_root.parents) + [local_root]
             for ignore_element in parent_local_configuration.ignore_all_errors:
@@ -461,23 +541,22 @@ class Configuration:
     def _check_read_local_configuration(self, path: str) -> None:
         if not os.path.exists(path):
             raise EnvironmentException(
-                "Local configuration path `{}` does not exist.".format(path)
+                f"Local configuration path `{path}` does not exist."
             )
 
         local_configuration = os.path.join(path, LOCAL_CONFIGURATION_FILE)
         if not os.path.exists(local_configuration):
             raise EnvironmentException(
-                "Local configuration directory `{}` does not contain "
-                "a `{}` file.".format(path, LOCAL_CONFIGURATION_FILE)
+                f"Local configuration directory `{path}` does not contain "
+                f"a `{LOCAL_CONFIGURATION_FILE}` file."
             )
 
         self._check_nested_configurations(path)
         self._read(local_configuration)
         if not self.source_directories and not self.targets:
             raise EnvironmentException(
-                "Local configuration `{}` does not specify any sources to type check.".format(
-                    local_configuration
-                )
+                f"Local configuration `{local_configuration}` does not specify "
+                " any sources to type check."
             )
 
     def _read(self, path: str) -> None:
@@ -487,17 +566,13 @@ class Configuration:
                 configuration_directory = os.path.dirname(os.path.realpath(path))
                 configuration = _ConfigurationFile(file)
 
-                source_directories = configuration.consume(
+                self.source_directories = configuration.consume(
                     "source_directories",
                     default=[],
                     current=self.source_directories,
                     print_on_success=True,
                     raise_on_override=True,
                 )
-                self.source_directories = [
-                    expand_relative_path(configuration_directory, directory)
-                    for directory in source_directories
-                ]
 
                 self.targets = configuration.consume(
                     "targets",
@@ -510,9 +585,7 @@ class Configuration:
                 if configuration.consume("disabled", default=False):
                     self.disabled = True
 
-                logger = configuration.consume("logger", current=self.logger)
-                if logger:
-                    self.logger = expand_relative_path(configuration_directory, logger)
+                self.logger = configuration.consume("logger", current=self.logger)
 
                 self.formatter = configuration.consume(
                     "formatter", current=self.formatter
@@ -523,20 +596,10 @@ class Configuration:
                 ignore_all_errors = configuration.consume(
                     "ignore_all_errors", default=[]
                 )
-                self.ignore_all_errors.extend(
-                    [
-                        expand_relative_path(root=configuration_directory, path=path)
-                        for path in ignore_all_errors
-                    ]
-                )
+                self.ignore_all_errors.extend(ignore_all_errors)
 
                 ignore_infer = configuration.consume("ignore_infer", default=[])
-                self.ignore_infer.extend(
-                    [
-                        expand_relative_path(root=configuration_directory, path=path)
-                        for path in ignore_infer
-                    ]
-                )
+                self.ignore_infer.extend(ignore_infer)
 
                 self.number_of_workers = int(
                     configuration.consume(
@@ -546,35 +609,32 @@ class Configuration:
 
                 binary = configuration.consume("binary", current=self._binary)
                 assert binary is None or isinstance(binary, str)
-                if binary is not None:
-                    binary = expand_relative_path(configuration_directory, binary)
                 self._binary = binary
 
-                buck_builder_binary = configuration.consume(
+                self._buck_builder_binary = configuration.consume(
                     "buck_builder_binary", current=self._buck_builder_binary
                 )
-                if buck_builder_binary is not None:
-                    self._buck_builder_binary = expand_relative_path(
-                        root=configuration_directory, path=buck_builder_binary
-                    )
 
                 additional_search_path = configuration.consume(
                     "search_path", default=[]
                 )
-
                 if isinstance(additional_search_path, list):
                     self._search_path.extend(
                         [
-                            SearchPathElement.expand(path, base=configuration_directory)
+                            SearchPathElement.expand(
+                                path,
+                                project_root=self.project_root,
+                                path_relative_to=configuration_directory,
+                            )
                             for path in additional_search_path
                         ]
                     )
                 else:
                     self._search_path.append(
-                        SearchPathElement(
-                            expand_relative_path(
-                                configuration_directory, additional_search_path
-                            )
+                        SearchPathElement.expand(
+                            additional_search_path,
+                            project_root=self.project_root,
+                            path_relative_to=configuration_directory,
                         )
                     )
 
@@ -586,8 +646,6 @@ class Configuration:
 
                 typeshed = configuration.consume("typeshed", current=self._typeshed)
                 assert typeshed is None or isinstance(typeshed, str)
-                if typeshed is not None:
-                    typeshed = expand_relative_path(configuration_directory, typeshed)
                 self._typeshed = typeshed
 
                 taint_models_path = configuration.consume("taint_models_path")
@@ -597,30 +655,16 @@ class Configuration:
                     or isinstance(taint_models_path, list)
                 )
                 if isinstance(taint_models_path, str):
-
-                    self.taint_models_path.append(
-                        expand_relative_path(configuration_directory, taint_models_path)
-                    )
+                    self.taint_models_path.append(taint_models_path)
                 elif isinstance(taint_models_path, list):
-                    self.taint_models_path.extend(
-                        [
-                            expand_relative_path(configuration_directory, path)
-                            for path in taint_models_path
-                        ]
-                    )
+                    self.taint_models_path.extend(taint_models_path)
 
                 excludes = configuration.consume("exclude", default=[])
+                # TODO(T71120969): Deprecate use of regexes for configuration paths.
                 if isinstance(excludes, list):
-                    self.excludes.extend(
-                        [
-                            expand_relative_path(configuration_directory, path)
-                            for path in excludes
-                        ]
-                    )
+                    self.excludes.extend(excludes)
                 else:
-                    self.excludes.append(
-                        expand_relative_path(configuration_directory, excludes)
-                    )
+                    self.excludes.append(excludes)
 
                 extensions = configuration.consume("extensions", default=[])
                 self.extensions.extend(extensions)
@@ -633,13 +677,17 @@ class Configuration:
                 if self._use_buck_builder is None:
                     self._use_buck_builder = use_buck_builder
 
+                use_buck_source_database = configuration.consume(
+                    "use_buck_source_database"
+                )
+                if self._use_buck_source_database is None:
+                    self._use_buck_source_database = use_buck_source_database
+
                 self.autocomplete = configuration.consume("autocomplete", default=False)
 
-                critical_files = configuration.consume("critical_files", default=[])
-                self.other_critical_files = [
-                    expand_relative_path(root=configuration_directory, path=path)
-                    for path in critical_files
-                ]
+                self.other_critical_files = configuration.consume(
+                    "critical_files", default=[]
+                )
 
                 # Warn on deprecated fields.
                 for deprecated_field in configuration._deprecated.keys():
@@ -652,15 +700,54 @@ class Configuration:
                         "Some configuration items were not recognized in "
                         "`{}`: {}".format(path, ", ".join(unused_keys))
                     )
+            self._expand_relative_paths(configuration_directory)
         except IOError:
             # To avoid TOCTTOU bugs, handle IOErrors here silently.
             # We error elsewhere if there weren't enough parameters passed into pyre.
-            pass
+            self._expand_relative_paths(configuration_directory)
         except json.JSONDecodeError as error:
             raise EnvironmentException(
                 "Configuration file at `{}` is invalid: {}.".format(path, str(error))
             )
 
+    def _expand_relative_paths(self, root: str) -> None:
+        self.source_directories = [
+            expand_relative_path(root, directory)
+            for directory in self.source_directories
+        ]
+
+        logger = self.logger
+        if logger:
+            self.logger = expand_relative_path(root, logger)
+
+        self.ignore_all_errors = [
+            expand_relative_path(root, path) for path in self.ignore_all_errors
+        ]
+
+        self.ignore_infer = [
+            expand_relative_path(root, path) for path in self.ignore_infer
+        ]
+
+        binary = self._binary
+        if binary:
+            self._binary = expand_relative_path(root, binary)
+
+        buck_builder_binary = self._buck_builder_binary
+        if buck_builder_binary is not None:
+            self._buck_builder_binary = expand_relative_path(root, buck_builder_binary)
+
+        typeshed = self._typeshed
+        if typeshed is not None:
+            self._typeshed = expand_relative_path(root, typeshed)
+
+        self.taint_models_path = [
+            expand_relative_path(root, path) for path in self.taint_models_path
+        ]
+
+        self.other_critical_files = [
+            expand_relative_path(root, path) for path in self.other_critical_files
+        ]
+
     def _resolve_versioned_paths(self) -> None:
         version_hash = self.version_hash
         if not version_hash:
@@ -685,9 +772,7 @@ class Configuration:
             self._binary = overriding_binary
             LOG.warning("Binary overridden with `%s`", self._binary)
         if not self._binary:
-            LOG.info(
-                "No binary specified, looking for `{}` in PATH".format(BINARY_NAME)
-            )
+            LOG.info(f"No binary specified, looking for `{BINARY_NAME}` in PATH")
             self._binary = shutil.which(BINARY_NAME)
             if not self._binary:
                 binary_candidate = os.path.join(
@@ -695,7 +780,7 @@ class Configuration:
                 )
                 self._binary = shutil.which(binary_candidate)
             if not self._binary:
-                LOG.warning("Could not find `{}` in PATH".format(BINARY_NAME))
+                LOG.warning(f"Could not find `{BINARY_NAME}` in PATH")
             else:
                 LOG.info("Found: `%s`", self._binary)
 
@@ -707,18 +792,12 @@ class Configuration:
 
         if not self._typeshed:
             LOG.info("No typeshed specified, looking for it")
-            self._typeshed = find_typeshed()
-            if not self._typeshed:
+            typeshed_path = find_typeshed()
+            if not typeshed_path:
                 LOG.warning("Could not find a suitable typeshed")
             else:
-                LOG.info("Found: `%s`", self._typeshed)
-
-        if not self._log_directory:
-            # TODO(T56191177): We should not start up a server at all if no configurations
-            # exist. Currently, we treat the cwd as the project root if no configurations
-            # exist. Instead, we should default logging to `tmp/.pyre` in the initial
-            # find_log_directory as well.
-            self._log_directory = "/tmp/.pyre"
+                LOG.info(f"Found: `{typeshed_path}`")
+            self._typeshed = str(typeshed_path) if typeshed_path is not None else None
 
     def _typeshed_has_obsolete_value(self) -> bool:
         (head, tail) = os.path.split(self.typeshed)
