@@ -1,4 +1,4 @@
-# Copyright (c) 2019-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -6,11 +6,10 @@
 
 import functools
 import logging
-import multiprocessing
+import logging.handlers
 import os
 import signal
 import sys
-from multiprocessing import Event
 from pathlib import Path
 from typing import Any, Dict, List, NamedTuple
 
@@ -26,6 +25,10 @@ Subscription = NamedTuple(
 )
 
 
+class WatchmanRestartedException(Exception):
+    pass
+
+
 def compute_pid_path(base_path: str, name: str) -> str:
     return str(Path(base_path, f"{name}.pid"))
 
@@ -34,7 +37,7 @@ class Subscriber(object):
     def __init__(self, base_path: str) -> None:
         self._base_path: str = base_path
         self._alive: bool = True
-        self._ready: multiprocessing.synchronize.Event = Event()
+        self._ready: bool = False
 
     @property
     def _name(self) -> str:
@@ -93,8 +96,13 @@ class Subscriber(object):
                 )
             ):
                 LOG.debug(f"Acquired lock on {lock_path}")
-                file_handler = logging.FileHandler(
-                    os.path.join(self._base_path, f"{self._name}.log"), mode="w"
+                file_handler = logging.handlers.RotatingFileHandler(
+                    os.path.join(self._base_path, f"{self._name}.log"),
+                    mode="a",
+                    # Keep at most 5 log files on disk
+                    backupCount=4,
+                    # Limit the size of each log file to 10MB
+                    maxBytes=10 * 1000 * 1000,
                 )
                 file_handler.setFormatter(
                     logging.Formatter("%(asctime)s %(levelname)s %(message)s")
@@ -119,15 +127,16 @@ class Subscriber(object):
                     # This call is blocking, which prevents this loop from burning CPU.
                     response = connection.receive()
                     if response.get("is_fresh_instance", False):
-                        # TODO: is_fresh_instance can occur at any time, not just the
-                        # first response. Ignoring the initial response is fine,
-                        # but if we receive a fresh instance response later we
-                        # should assume that all files may have been changed.
-                        root = response.get("root", "<no-root-found>")
-                        LOG.info(f"Ignoring is_fresh_instance message for {root}")
+                        if not self._ready:
+                            root = response.get("root", "<no-root-found>")
+                            self._ready = True
+                            LOG.info(
+                                f"Ignoring initial is_fresh_instance message for {root}"
+                            )
+                        else:
+                            raise WatchmanRestartedException()
                     else:
                         self._handle_response(response)
-                    self._ready.set()  # At least one message has been received.
         finally:
             LOG.info("Cleaning up lock and pid files before exiting.")
             remove_if_exists(lock_path)
@@ -147,12 +156,12 @@ class Subscriber(object):
                     os.close(sys.stdout.fileno())
                     os.close(sys.stderr.fileno())
                     self._run()
-                    sys.exit(0)
+                    os._exit(0)
                 except Exception as exception:
                     LOG.info(f"Not running {self._name} due to {exception}")
-                    sys.exit(1)
+                    os._exit(1)
             else:
-                sys.exit(0)
+                os._exit(0)
 
 
 def stop_subscriptions(base_path: str, subscriber_name: str) -> None:
