@@ -1,4 +1,7 @@
-# (c) Facebook, Inc. and its affiliates. Confidential and proprietary.
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 import logging
 import os
@@ -6,27 +9,30 @@ from typing import Optional
 
 import sqlalchemy
 from flask import Flask, send_from_directory
-from flask_cors import CORS
 from flask_graphql import GraphQLView
 from sqlalchemy.orm import Session, scoped_session, sessionmaker
 
+from .db import DB
 from .models import Base
 from .schema import schema
 
 
+logging.basicConfig(
+    format="%(asctime)s [%(levelname)s] %(message)s", level=logging.DEBUG
+)
+LOG = logging.getLogger(__name__)
+
+
 application = Flask(
-    __name__,
-    static_folder=os.path.join(os.path.dirname(__file__), "..", "frontend", "build"),
+    __name__, static_folder=os.path.join(os.path.dirname(__file__), "frontend", "build")
 )
-# TODO(T58070180): we should remove the cross origin policy.
-CORS(application)
-application.debug = True
 
 session: Optional[Session] = None
 
 
 @application.teardown_appcontext
 def shutdown_session(exception=None):
+
     if session is not None:
         session.remove()
 
@@ -34,20 +40,23 @@ def shutdown_session(exception=None):
 @application.route("/", defaults={"path": ""})
 @application.route("/<path:path>")
 def serve(path):
+    LOG.info(f"Serving `{path}`...")
     if path != "" and os.path.exists(application.static_folder + "/" + path):
+        LOG.info("Found static resource.")
         return send_from_directory(application.static_folder, path)
     else:
-        logging.error(application.static_folder)
+        LOG.info("Resource not found. Falling back to `index.html`")
         return send_from_directory(application.static_folder, "index.html")
 
 
-def start_app(database):
+def start_server(database: DB, debug: bool, static_resources: Optional[str]) -> None:
     engine = sqlalchemy.create_engine(
         sqlalchemy.engine.url.URL("sqlite", database=database.dbname),
         echo=False,
         poolclass=None,
     )
     session = scoped_session(sessionmaker(bind=engine))
+    # pyre-ignore
     Base.query = session.query_property()
 
     application.add_url_rule(
@@ -59,5 +68,6 @@ def start_app(database):
             get_context=lambda: {"session": session},
         ),
     )
-
-    application.run()
+    if static_resources:
+        application.static_folder = static_resources
+    application.run(debug=debug)
