@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -1320,7 +1320,7 @@ class PrimaryKeyGenerator(PrimaryKeyGeneratorBase):
 
 def create(db: DB):
     try:
-        Base.metadata.create_all(db.engine)  # pyre-ignore [16]
+        Base.metadata.create_all(db.engine)
     except NoSuchTableError:
         pass
 
