@@ -1,3 +1,8 @@
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
+
 from collections import defaultdict
 from enum import Enum
 from typing import Dict, List, Optional, Set, Tuple, Union
@@ -7,6 +12,7 @@ from sqlalchemy.orm.query import Query
 from sqlalchemy.sql.expression import or_
 
 from .models import (
+    DBID,
     Issue,
     IssueInstance,
     IssueInstanceSharedTextAssoc,
@@ -34,7 +40,7 @@ class Filter(Enum):
 
 
 class IssueQueryBuilder:
-    def __init__(self, current_run_id: int):
+    def __init__(self, current_run_id: Union[DBID, int]):
         self._session = None
         self.current_run_id = current_run_id
         self.issue_filters: Dict[
@@ -53,7 +59,7 @@ class IssueQueryBuilder:
         return self
 
     def get(self) -> List:
-        query = self._get_session_query(self._session)
+        query = self.get_session_query(self._session)
         for filter_type, filter_conditions in self.issue_filters.items():
             if filter_type == Filter.codes:
                 column = Issue.code
@@ -71,9 +77,9 @@ class IssueQueryBuilder:
                     filter_type == Filter.trace_length_to_sources
                     or filter_type == Filter.trace_length_to_sinks
                 ):
-                    if filter_condition[0]:
+                    if filter_condition[0] is not None:
                         query = query.filter(column >= filter_condition[0])
-                    if filter_condition[1]:
+                    if filter_condition[1] is not None:
                         query = query.filter(column <= filter_condition[1])
                 else:
                     if not filter_condition:
@@ -103,7 +109,7 @@ class IssueQueryBuilder:
                 raise Exception(f"Invalid filter type provided: {filter_type}")
 
             features_list = [
-                self._get_leaves_issue_instance(
+                self.get_leaves_issue_instance(
                     self._session, int(issue.id), SharedTextKind.FEATURE
                 )
                 for issue in issues
@@ -157,7 +163,7 @@ class IssueQueryBuilder:
 
     def sources(self, issues) -> List[Set[str]]:
         return [
-            self._get_leaves_issue_instance(
+            self.get_leaves_issue_instance(
                 self._session, int(issue.id), SharedTextKind.SINK
             )
             for issue in issues
@@ -165,7 +171,7 @@ class IssueQueryBuilder:
 
     def sinks(self, issues) -> List[Set[str]]:
         return [
-            self._get_leaves_issue_instance(
+            self.get_leaves_issue_instance(
                 self._session, int(issue.id), SharedTextKind.SOURCE
             )
             for issue in issues
@@ -173,13 +179,13 @@ class IssueQueryBuilder:
 
     def features(self, issues) -> List[Set[str]]:
         return [
-            self._get_leaves_issue_instance(
+            self.get_leaves_issue_instance(
                 self._session, int(issue.id), SharedTextKind.FEATURE
             )
             for issue in issues
         ]
 
-    def _get_session_query(self, session: Session) -> Query:
+    def get_session_query(self, session: Session) -> Query:
         return (
             session.query(
                 IssueInstance.id,
@@ -196,7 +202,7 @@ class IssueQueryBuilder:
             .join(CallableText, CallableText.id == IssueInstance.callable_id)
         )
 
-    def _get_leaves_issue_instance(
+    def get_leaves_issue_instance(
         self, session: Session, issue_instance_id: int, kind: SharedTextKind
     ) -> Set[str]:
         message_ids = [
