@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -11,7 +11,9 @@ import subprocess
 import sys
 from collections import defaultdict
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Optional, Tuple
+from typing import Any, Dict, Iterator, List, Optional, Tuple, Union, cast
+
+import libcst
 
 from . import UserError, ast
 
@@ -20,6 +22,39 @@ LOG: logging.Logger = logging.getLogger(__name__)
 MAX_LINES_PER_FIXME: int = 4
 
 
+class LineBreakTransformer(libcst.CSTTransformer):
+    def leave_SimpleWhitespace(
+        self,
+        original_node: libcst.SimpleWhitespace,
+        updated_node: libcst.SimpleWhitespace,
+    ) -> Union[libcst.SimpleWhitespace, libcst.ParenthesizedWhitespace]:
+        whitespace = original_node.value.replace("\\", "")
+        if "\n" in whitespace:
+            first_line = libcst.TrailingWhitespace(
+                whitespace=libcst.SimpleWhitespace(
+                    value=whitespace.split("\n")[0].rstrip()
+                ),
+                comment=None,
+                newline=libcst.Newline(),
+            )
+            last_line = libcst.SimpleWhitespace(value=whitespace.split("\n")[1])
+            return libcst.ParenthesizedWhitespace(
+                first_line=first_line, empty_lines=[], indent=True, last_line=last_line
+            )
+        return updated_node
+
+    def leave_Assign(
+        self, original_node: libcst.Assign, updated_node: libcst.Assign
+    ) -> libcst.Assign:
+        assign_value = updated_node.value
+        if hasattr(assign_value, "lpar"):
+            parenthesized_value = assign_value.with_changes(
+                lpar=[libcst.LeftParen()], rpar=[libcst.RightParen()]
+            )
+            return updated_node.with_changes(value=parenthesized_value)
+        return updated_node
+
+
 class PartialErrorSuppression(Exception):
     def __init__(self, message: str, unsuppressed_paths: List[str]) -> None:
         super().__init__(message)
@@ -64,8 +99,6 @@ class Errors:
         self.errors: List[Dict[str, Any]] = errors
         self.error_iterator: Iterator[
             Tuple[str, Iterator[Dict[str, Any]]]
-            # pyre-fixme[6]: Expected `(_T) -> _SupportsLessThan` for 2nd param but got
-            #  `(error: Dict[str, typing.Any]) -> str`.
         ] = itertools.groupby(sorted(errors, key=error_path), error_path)
         self.length: int = len(errors)
 
@@ -108,7 +141,7 @@ class Errors:
                 path.write_text(output)
             except SkippingGeneratedFileException:
                 LOG.warning(f"Skipping generated file at {path_to_suppress}")
-            except ast.UnstableAST:
+            except (ast.UnstableAST, SyntaxError):
                 unsuppressed_paths.append(path_to_suppress)
 
         if unsuppressed_paths:
@@ -213,6 +246,30 @@ def _remove_comment_preamble(lines: List[str]) -> None:
             return
 
 
+def _add_error_to_line_break_block(lines: List[str], errors: List[List[str]]) -> None:
+    # Gather unbroken lines.
+    line_break_block = [lines.pop() for _ in range(0, len(errors))]
+    line_break_block.reverse()
+
+    # Transform line break block to use parenthesis.
+    indent = len(line_break_block[0]) - len(line_break_block[0].lstrip())
+    line_break_block = [line[indent:] for line in line_break_block]
+    statement = "\n".join(line_break_block)
+    transformed_statement = libcst.Module([]).code_for_node(
+        cast(
+            libcst.CSTNode,
+            libcst.parse_statement(statement).visit(LineBreakTransformer()),
+        )
+    )
+    transformed_lines = transformed_statement.split("\n")
+    transformed_lines = [" " * indent + line for line in transformed_lines]
+
+    # Add to lines.
+    for line, comment in zip(transformed_lines, errors):
+        lines.extend(comment)
+        lines.append(line)
+
+
 def _split_across_lines(
     comment: str, indent: int, max_line_length: Optional[int]
 ) -> List[str]:
@@ -263,6 +320,7 @@ def _suppress_errors(
     # Replace lines in file.
     new_lines = []
     removing_pyre_comments = False
+    line_break_block_errors: List[List[str]] = []
     for index, line in enumerate(lines):
         if removing_pyre_comments:
             stripped = line.lstrip()
@@ -273,11 +331,8 @@ def _suppress_errors(
             else:
                 removing_pyre_comments = False
         number = index + 1
-        if number not in errors:
-            new_lines.append(line)
-            continue
-
-        if any(error["code"] == "0" for error in errors[number]):
+        relevant_errors = errors[number] if number in errors else []
+        if any(error["code"] == "0" for error in relevant_errors):
             # Handle unused ignores.
             replacement = re.sub(r"# pyre-(ignore|fixme).*$", "", line).rstrip()
             if replacement == "":
@@ -288,7 +343,7 @@ def _suppress_errors(
                 line = replacement
 
         comments = []
-        for error in errors[number]:
+        for error in relevant_errors:
             if error["code"] == "0":
                 continue
             indent = len(line) - len(line.lstrip(" "))
@@ -296,7 +351,6 @@ def _suppress_errors(
             comment = "{}# pyre-fixme[{}]: {}".format(
                 " " * indent, error["code"], description
             )
-
             if not max_line_length or len(comment) <= max_line_length:
                 comments.append(comment)
             else:
@@ -309,6 +363,18 @@ def _suppress_errors(
                 else:
                     comments.extend(split_comment_lines)
 
+        if len(line_break_block_errors) > 0 and not line.endswith("\\"):
+            # Handle error suppressions in line break block
+            line_break_block_errors.append(comments)
+            new_lines.append(line)
+            _add_error_to_line_break_block(new_lines, line_break_block_errors)
+            line_break_block_errors = []
+            continue
+
+        if line.endswith("\\"):
+            line_break_block_errors.append(comments)
+            comments = []
+
         LOG.info(
             "Adding comment%s on line %d: %s",
             "s" if len(comments) > 1 else "",
