@@ -1,4 +1,4 @@
-# Copyright (c) 2016-present, Facebook, Inc.
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,7 +9,11 @@ from typing import List, Optional
 
 
 class Repository:
-    MIGRATION_SUMMARY: str = "Migrating buck integration to use configurations"
+    MIGRATION_SUMMARY: str = (
+        "Migrating buck integration to use configurations.\n "
+        "For more information about this migration, please see: "
+        "https://fb.workplace.com/groups/295311271085134/permalink/552700215346237/"
+    )
 
     def commit_message(self, title: str, summary_override: Optional[str] = None) -> str:
         return ""
@@ -35,6 +39,7 @@ class Repository:
         pass
 
     def format(self) -> bool:
+        # pyre-fixme[7]: Expected `bool` but got implicit return value of `None`.
         pass
 
     def force_format(self, paths: List[str]) -> None:
